/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
/* POSIX directory manipulation */
#include <sys/types.h>
#include <dirent.h>
/* Date manipulation */
#include <time.h>
/* String manipulation */
#include <string.h>
#include "mystring.h"
/* Seasons manipulation */
#include "seasons.h"
/* OS-specific functions */
#include "shell.h"
#include "myconsole.h"

#define PROGRAM_NAME "WallpaperSeasonsSwitcher"
#define PROGRAM_VERSION "1.0"

#define WALLPAPERS_DIR ".\\Wallpapers"


// Define whether the console is displayed or not
int is_console = FALSE;

int SwitchWallpaper()
{
  	time_t timestamp;                       /* Current date and time */
	struct tm *t;                           /* Struct holding time informations */
    enum season c_season;                   /* Current season */
	enum season c_wall_season;              /* Current wallpaper season */
	char season_name[10];

    char c_dir[MAX_PATH];                   /* Current directory */
    char c_wall_filename[MAX_PATH];         /* Current wallpaper */
	char new_wall_filename[MAX_PATH];       /* Filename of the new wallpaper */
	char new_wall_filename_ex[MAX_PATH];    /* Complete resolved filename to the new wallpaper */

	/* Read current date and time */
	timestamp = time (NULL);
	t = gmtime (&timestamp);

#ifdef DEBUG
	Error("Current year day : %i\n", t->tm_yday);
#endif

	/* Set current season */
	if (sSpring <= t->tm_yday && t->tm_yday < sSummer) {
		c_season = sSpring;
	} else if (sSummer <= t->tm_yday && t->tm_yday < sAutumn) {
		c_season = sSummer;
	} else if (sAutumn <= t->tm_yday && t->tm_yday < sWinter) {
		c_season = sAutumn;
	} else {
		c_season = sWinter;
	}

#ifdef DEBUG
	Error("Current season : %s\n", getSeasonName (season_name, c_season) );
#endif

	/* Read current wallpaper filename */
    if (GetCurrentWallpaper(c_wall_filename, MAX_PATH) != ERROR_SUCCESS) {
        Error("Error : the current wallpaper name cannot be retrieved.\n");
        return 1;
    }

	/* Read current wallpaper season */
	if (stripos (c_wall_filename, "Spring") != -1) {
		c_wall_season = sSpring;
	} else if (stripos (c_wall_filename, "Summer") != -1) {
		c_wall_season = sSummer;
	} else if (stripos (c_wall_filename, "Autumn") != -1) {
		c_wall_season = sAutumn;
	} else if (stripos (c_wall_filename, "Winter") != -1) {
		c_wall_season = sWinter;
	} else {
		Error("Error : the wallpaper \"%s\" does not seem to be a seasoned wallpaper.\n", c_wall_filename);
		return 1;
	}

#ifdef DEBUG
	Error("Current wallpaper season : %s\n", getSeasonName (season_name, c_wall_season) );
#endif

	/* Check if we have to change the wallpaper */
	if (c_season == c_wall_season) {
		Error("The wallpaper is already matching the current season.\n");
		return 0;
	}

    /* Read current exe directory */
    GetExeDirectory(c_dir, MAX_PATH);

	/* Search matching wallpaper */
	char wallpaper_dir[MAX_PATH];
	strncpy(wallpaper_dir, c_dir, MAX_PATH);

	struct dirent *file;
	DIR *folder;
	folder = opendir (strncat(wallpaper_dir, WALLPAPERS_DIR, MAX_PATH));
	while ( (file = readdir (folder) ) ) {
#ifdef DEBUG
		Error("File: %s\n", file->d_name);
#endif
		if (stripos (file->d_name, getSeasonName (season_name, c_season) ) != -1
            && stripos(file->d_name, ".jpg")) {
			strcpy (new_wall_filename, file->d_name);
			break;
		}
	}
	closedir (folder);

    if (strlen (new_wall_filename) <= 1) {
		Error("Error : impossible to find a wallpaper containing the word \"%s\".\n", getSeasonName (season_name, c_season) );
		return 1;
	}

	/* Get the full path to the new wallpaper */
	sprintf(new_wall_filename_ex, "%s\\%s\\%s", c_dir, WALLPAPERS_DIR, new_wall_filename);

    /* Set the new wallpaper */
    if (SetWallpaper(new_wall_filename_ex, TRUE) != ERROR_SUCCESS) {
        Error("Error : error while changing the wallpaper to \"%s\".\n", new_wall_filename_ex);
        return 1;
    }

#ifdef DEBUG
    Error("Wallpaper changed.\n");
#endif

	/* End */
	return 0;
}

void CreateConsole()
{
    if(is_console == FALSE) {
        RedirectIOToConsole();
        is_console = TRUE;
    }
}

void PrintHelp()
{
    printf("%s v%s\n", PROGRAM_NAME, PROGRAM_VERSION);
    printf("Switch through a set of wallpapers according to the current season\n\n");
    printf("Command-line switches :\n");
    printf("\t/console, /C\t\tDisplay the message console\n");
    printf("\t/register, /R\t\tExecute the program on Windows startup\n");
    printf("\t/unregister, /U\t\tUnregister the program from startup\n");
    printf("\t/register-all, /RA\tExecute the program on Windows startup for all users\n");
    printf("\t/unregister-all, /UA\tUnregister the program from startup for all users\n");
    printf("\t/help, /?\t\tDisplay this help message\n");

}

int ProcessArguments(int argc, char *argv[])
{
    int i;                          /* Arguments counter */
    int result;                     /* Commands result */
    char exe_filename[MAX_PATH];    /* Full path to the current exe */
    int return_state = PROCESS_CONTINUE; /* The return value */

    /* Get absolute program path (as argv[0] may be relative) */
    GetExeFilename(exe_filename, MAX_PATH);

    /* Foreach command line argument */
    for (i = 1; i < argc; i++) {

        if (strcmp(argv[i], "/console") == 0 || strcmp(argv[i], "/C") == 0) {
#ifdef DEBUG
            printf("Enable console output.");
#endif
            CreateConsole();
        } else

        /* "register" : launch the program on system startup */
        if (strcmp(argv[i], "/register") == 0 || strcmp(argv[i], "/R") == 0) {
#ifdef DEBUG
            printf("Register program on system startup.\n");
#endif
            result = RegisterStartup(PROGRAM_NAME, exe_filename, FALSE);
            if (result == FALSE) {
                printf("Error : cannot register the program to run on system startup.\n");
                return 1;
            }
            return_state = PROCESS_ABORT;
        } else

        /* "unregister" : remove the launch of the program on system startup */
        if (strcmp(argv[i], "/unregister") == 0 || strcmp(argv[i], "/U") == 0) {
#ifdef DEBUG
            printf("Unregister program from system startup.\n");
#endif
            result = UnregisterStartup(PROGRAM_NAME, FALSE);
            if (result == FALSE) {
                printf("Error : cannot register the program to run on system startup.\n");
                return 1;
            }
            return_state = PROCESS_ABORT;
        } else

        /* "register-all" : launch the program on system startup for all users */
        if (strcmp(argv[i], "/register-all") == 0 || strcmp(argv[i], "/RA") == 0) {
#ifdef DEBUG
            printf("Register program on system startup.\n");
#endif
            result = RegisterStartup(PROGRAM_NAME, exe_filename, TRUE);
            if (result == FALSE) {
                printf("Error : cannot register the program to run on system startup.\n");
                return 1;
            }
            return_state = PROCESS_ABORT;
        } else

        /* "unregister-all" : remove the launch of the program on system startup for all users*/
        if (strcmp(argv[i], "/unregister-all") == 0 || strcmp(argv[i], "/UA") == 0) {
#ifdef DEBUG
            printf("Unregister program from system startup.\n");
#endif
            result = UnregisterStartup(PROGRAM_NAME, TRUE);
            if (result == FALSE) {
                printf("Error : cannot register the program to run on system startup.\n");
                return 1;
            }
            return_state = PROCESS_ABORT;
        } else

        /* "help" : print command line infos */
        if (strcmp(argv[i], "/?") == 0 || strcmp(argv[i], "/help") == 0) {
            CreateConsole();
            PrintHelp();
            return_state = PROCESS_ABORT;
        }

        /* Invalid argument */
        else {
            CreateConsole();
            printf("Error: invalid argument \"%s\"\n\n", argv[i]);
            PrintHelp();
            return_state = PROCESS_ABORT;
        }
    }

    return return_state;
}

int main(int argc, char *argv[])
{
    int process = PROCESS_CONTINUE;
    int result;

    /* Process command-line arguments */
    if (argc > 1) {
        process = ProcessArguments(argc, argv);
    }

    /* If the command-line arguments didn't cancel the process, continue
     * and change the wallpaper */
    if (process == PROCESS_CONTINUE) {
        result = SwitchWallpaper(argv[0]);
    } else {
        result = process;
    }

    // Pause the console if displayed
    if (is_console) {
        system("PAUSE");
    }

    return result;
}
