#include "shell.h"

/**
 * Execute the given command-line, wait for the command to return,
 * and return the result.
 */
DWORD ExecWait(char * commandLine)
{
    /* Execute the commant */
    STARTUPINFO          si = { sizeof(si) };
    PROCESS_INFORMATION  pi;
    DWORD result;
    if (!CreateProcess(NULL, commandLine, NULL, NULL, FALSE, CREATE_NO_WINDOW, NULL, NULL, &si, &pi)) {
        return -1;
    }

    /* Wait for the command to end */
    WaitForSingleObject(pi.hProcess, INFINITE);

    /* Get the command result */
    GetExitCodeProcess(pi.hProcess,&result);

    /* Cleanup */
    CloseHandle( pi.hProcess );
    CloseHandle( pi.hThread );

    return result;
}

/**
 * Get the path to the executable (i.e. not the working directory)
 */
long GetExeDirectory(char * filename, int max_length)
{
    char exe_filename[MAX_PATH];
    char * delimiter;
    int del_index;
    long result;

    // Get the executable filename
    result = GetExeFilename(exe_filename, MAX_PATH);
    if (exe_filename == NULL) {
        return result;
    }

    // Figure out the path from the exe filename
    delimiter = strrchr(exe_filename, '\\');
    if (delimiter == NULL) {
        return ERROR_BAD_COMMAND;
    }

    del_index = delimiter - exe_filename + 1;
    if (del_index >= max_length) {
        return ERROR_BAD_LENGTH;
    }
    strncpy(filename, exe_filename, del_index);
    filename[del_index] = '\0';

    return ERROR_SUCCESS;
}

/**
 * Returns the true exe filename (ie not argv[0])
 */
long GetExeFilename(char * filename, int max_length)
{
    return GetModuleFileName(NULL, filename, max_length);
}


/**
 * Get the filename of the current wallpaper.
 */
long GetCurrentWallpaper(char * filename, int max_length)
{
    return ReadRegStr(filename, max_length, HKEY_CURRENT_USER, "Control Panel\\Desktop", "ConvertedWallpaper");
}

/**
 * Convert the given file to bitmap, and provide the name of the
 * converted file.
 */
long ConvertJpgToBmp(const char * filename, char * converted_filename)
{
    char jpg_command[MAX_PATH * 2] = "jpgtobmp.exe";    /* Command to the convert to jpg exe */
    char *search_result;                    /* Temp var for strchr() */

    /* Concatenate command and filename */
    if (strlen(jpg_command) + 1 + strlen(filename) > MAX_PATH * 2) {
        return ERROR_BUFFER_OVERFLOW;
    }
    sprintf(jpg_command, "%s \"%s\"", jpg_command, filename);

	/* Run conversion */
	if (ExecWait(jpg_command) != ERROR_SUCCESS) {
        return ERROR_INVALID_COMMAND_LINE;
    }

    /* Build the filename of the converted bitmap */
    search_result = strrchr(filename, '.');
    if (search_result == NULL) {
        return ERROR_NOACCESS;
    }
    int converted_length = strlen(filename) - strlen(search_result);
    strncpy(converted_filename, filename, converted_length);
    converted_filename[converted_length] = '\0';
    strcat(converted_filename, ".BMP");

    return ERROR_SUCCESS;
}

/**
 * Replace the current wallpaper by the given file, and optionnaly
 * convert it to bitmap before, using ConvertJpgToBmp()
 */
long SetWallpaper(const char * filename, BOOL convertToBmp)
{
    char converted_filename[MAX_PATH];
    char tmp_wall_filename[MAX_PATH];       /* Temporary Windows bitmap of the wallpaper */
    long result;

    /* Convert the new wallpaper to jpg */
    if (convertToBmp == TRUE) {
        result = ConvertJpgToBmp(filename, converted_filename);
        if (result != ERROR_SUCCESS) {
            return result;
        }
    } else {
        strcpy(converted_filename, filename);
    }

	/* Copy the wallpaper in a temporary folder */
	ExpandEnvironmentStrings("%USERPROFILE%\\Local Settings\\Application Data\\Wallpaper1.bmp", tmp_wall_filename, MAX_PATH + 1);
    if (MoveFileEx(converted_filename, tmp_wall_filename, MOVEFILE_COPY_ALLOWED | MOVEFILE_REPLACE_EXISTING) == FALSE) {
        return 1;
    }

	/* Change the wallpaper */
    SystemParametersInfo(SPI_SETDESKWALLPAPER, 0, tmp_wall_filename, SPIF_SENDWININICHANGE | SPIF_UPDATEINIFILE);

    /* Update registry settings */
    result = WriteRegStr(HKEY_CURRENT_USER, "Control Panel\\Desktop", "ConvertedWallpaper", filename);
    if (result != ERROR_SUCCESS) {
        return result;
    }

    return ERROR_SUCCESS;
}

/**
 * Register the given command to be run on system startup.
 */
BOOL RegisterStartup(char * program_name, char *exe_filename, BOOL allUsers)
{
    int result;
    HKEY rootKey;
    if (allUsers == TRUE) {
        rootKey = HKEY_LOCAL_MACHINE;
    } else {
        rootKey = HKEY_CURRENT_USER;
    }

    // Register the exe_filename on startup
    result = WriteRegStr(rootKey,
                         "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Run",
                         program_name,
                         exe_filename);

    return (result == ERROR_SUCCESS) ? TRUE : FALSE;
}

/**
 * Unregister the given command from being run on system startup.
 */
BOOL UnregisterStartup(char * program_name, BOOL allUsers)
{
    int result;
    HKEY rootKey;
    if (allUsers == TRUE) {
        rootKey = HKEY_LOCAL_MACHINE;
    } else {
        rootKey = HKEY_CURRENT_USER;
    }

    result = DeleteRegValue(rootKey,
                   "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Run",
                   program_name);

    return (result == ERROR_SUCCESS || result == ERROR_FILE_NOT_FOUND) ? TRUE : FALSE;
}

/**
 * Check if a given program and command-line is registered.
 */
BOOL IsRegistered(char * program_name, char * exe_filename)
{
    char result[MAX_PATH];

    ReadRegStr(result, MAX_PATH, HKEY_CURRENT_USER, "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Run", program_name);
    if (strcmp(result, exe_filename) == 0) {
        return TRUE;
    } else {
        return FALSE;
    }
}
